# part of Tk 8.4.5 menu.tcl library file.
# MenuUnpost procedure modified by r0lZ to fix a Tk bug with tearoff menus.
#
# Original Tcl/Tk copyright notice:
#
# RCS: @(#) $Id: menu.tcl,v 1.18 2002/08/31 06:12:28 das Exp $
#
# Copyright (c) 1992-1994 The Regents of the University of California.
# Copyright (c) 1994-1997 Sun Microsystems, Inc.
# Copyright (c) 1998-1999 by Scriptics Corporation.
#
# See the file "license.terms" for information on usage and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.

proc ::tk::MenuUnpost menu {
    global tcl_platform
    variable ::tk::Priv
    set mb $Priv(postedMb)

    # Restore focus right away (otherwise X will take focus away when
    # the menu is unmapped and under some window managers (e.g. olvwm)
    # we'll lose the focus completely).

    catch {focus $Priv(focus)}
    set Priv(focus) ""

    # Unpost menu(s) and restore some stuff that's dependent on
    # what was posted.

    catch {
	if {[string compare $mb ""]} {
	    set menu [$mb cget -menu]
	    $menu unpost
	    set Priv(postedMb) {}
	    $mb configure -cursor $Priv(cursor)
	    $mb configure -relief $Priv(relief)
	} elseif {[string compare $Priv(popup) ""]} {
	    $Priv(popup) unpost
	    set Priv(popup) {}
	} elseif {[string compare [$menu cget -type] "menubar"] \
		&& [string compare [$menu cget -type] "tearoff"]} {
	    # We're in a cascaded sub-menu from a torn-off menu or popup.
	    # Unpost all the menus up to the toplevel one (but not
	    # including the top-level torn-off one) and deactivate the
	    # top-level torn off menu if there is one.

	    while {1} {
		set parent [winfo parent $menu]
		if {[string compare [winfo class $parent] "Menu"] \
			|| ![winfo ismapped $parent]} {
		    break
		}
		$parent activate none
		$parent postcascade none
		GenerateMenuSelect $parent
		set type [$parent cget -type]
		if {[string equal $type "menubar"] || \
			[string equal $type "tearoff"]} {
		    break
		}
		set menu $parent
	    }
	    if {[string compare [$menu cget -type] "menubar"]} {
		$menu unpost
	    }
	}
    }

    if {($Priv(tearoff) != 0) || $Priv(menuBar) ne ""} {
    	# Release grab, if any, and restore the previous grab, if there
    	# was one.
	if {[string compare $menu ""]} {
	    set grab [grab current $menu]
	    if {[string compare $grab ""]} {
		grab release $grab
	    }
	}
	RestoreOldGrab
	if {$Priv(menuBar) ne ""} {
	    # Tk bug fixed by r0lZ: added the catch
	    catch {$Priv(menuBar) configure -cursor $Priv(cursor)}
	    set Priv(menuBar) {}
	}
	if {[tk windowingsystem] ne "x11"} {
	    set Priv(tearoff) 0
	}
    }
}

# ::tk::PostOverPoint --
# This procedure posts a given menu such that a given entry in the
# menu is centered over a given point in the root window.  It also
# activates the given entry.
#
# Arguments:
# menu -		Menu to post.
# x, y -		Root coordinates of point.
# entry -		Index of entry within menu to center over (x,y).
#			If omitted or specified as {}, then the menu's
#			upper-left corner goes at (x,y).
#
# Modified by r0lZ to fix bad position on Windows platforms

proc ::tk::PostOverPoint {menu x y {entry {}}}  {
    global tcl_platform
    
    if {[string compare $entry {}]} {
	if {$entry == [$menu index last]} {
	    incr y [expr {-([$menu yposition $entry] \
		    + [winfo reqheight $menu])/2}]
	} else {
	    incr y [expr {-([$menu yposition $entry] \
		    + [$menu yposition [expr {$entry+1}]])/2}]
	}
	incr x [expr {-[winfo reqwidth $menu]/2}]
    }
    if {$tcl_platform(platform) == "windows"} {
	# We need to fix some problems with menu posting on Windows.
	# bug fixed by r0lZ: -4 was missing
	set yoffset [expr {[winfo screenheight $menu] - 4 \
		- $y - [winfo reqheight $menu]}]
	if {$yoffset < 0} {
	    # The bottom of the menu is offscreen, so adjust upwards
	    incr y $yoffset
	}
	# If we're off the top of the screen (either because we were
	# originally or because we just adjusted too far upwards),
	# then make the menu popup on the top edge.
	if {$y < 0} {
	    set y 0
	}

	# Added by r0lZ: all stuff to fix the X position
	set xoffset [expr {[winfo screenwidth $menu] - 8 \
		- $x - [winfo reqwidth $menu]}]
	if {$xoffset < 0} {
	    # The bottom of the menu is offscreen, so adjust upwards
	    incr x $xoffset
	}
	if {$x < 0} {
	    set x 0
	}
    }
    $menu post $x $y
    if {$entry ne "" && [$menu entrycget $entry -state] ne "disabled"} {
	$menu activate $entry
	GenerateMenuSelect $menu
    }
}


# part of Tk 8.4.5 -- button.tcl --
# ButtonLeave proc modified by r0lZ to avoid a bug when the button has an image
#
# This file defines the default bindings for Tk label, button,
# checkbutton, and radiobutton widgets and provides procedures
# that help in implementing those bindings.
#
# RCS: @(#) $Id: button.tcl,v 1.17 2002/09/04 02:05:52 hobbs Exp $
#
# Copyright (c) 1992-1994 The Regents of the University of California.
# Copyright (c) 1994-1996 Sun Microsystems, Inc.
# Copyright (c) 2002 ActiveState Corporation.
#
# See the file "license.terms" for information on usage and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#

# ::tk::ButtonLeave --
# The procedure below is invoked when the mouse pointer leaves a
# button widget.  It changes the state of the button back to inactive.
# Restore any modified relief too.
#
# Arguments:
# w -		The name of the widget.

proc ::tk::ButtonLeave w {
    variable ::tk::Priv
    if {[$w cget -state] ne "disabled"} {
	catch {$w configure -state normal}
    }

    # Restore the original button relief if it was changed by Tk.
    # That is signaled by the existence of Priv($w,prelief).

    if {[info exists Priv($w,relief)]} {
	if {[info exists Priv($w,prelief)] && \
		$Priv($w,prelief) eq [$w cget -relief]} {
	    catch {$w configure -relief $Priv($w,relief)}
	}
	unset -nocomplain Priv($w,relief) Priv($w,prelief)
    }

    set Priv(window) ""
}

# ::tk::ButtonEnter --
# The procedure below is invoked when the mouse pointer enters a
# button widget.  It records the button we're in and changes the
# state of the button to active unless the button is disabled.
#
# Arguments:
# w -		The name of the widget.

proc ::tk::ButtonEnter w {
    variable ::tk::Priv
    if {[winfo exists $w] && [$w cget -state] ne "disabled"} {

		# If the mouse button is down, set the relief to sunken on entry.
		# Overwise, if there's an -overrelief value, set the relief to that.
	
		set Priv($w,relief) [$w cget -relief]
		if {$Priv(buttonWindow) eq $w} {
		    catch {$w configure -relief sunken -state active}
		    set Priv($w,prelief) sunken
		} elseif {[set over [$w cget -overrelief]] ne ""} {
		    catch {$w configure -relief $over}
		    set Priv($w,prelief) $over
		}
    }
    set Priv(window) $w
}

# ::tk::ButtonDown --
# The procedure below is invoked when the mouse button is pressed in
# a button widget.  It records the fact that the mouse is in the button,
# saves the button's relief so it can be restored later, and changes
# the relief to sunken.
#
# Arguments:
# w -		The name of the widget.

proc ::tk::ButtonDown w {
    variable ::tk::Priv

    # Only save the button's relief if it does not yet exist.  If there
    # is an overrelief setting, Priv($w,relief) will already have been set,
    # and the current value of the -relief option will be incorrect.

	catch {
	    if {![info exists Priv($w,relief)]} {
			set Priv($w,relief) [$w cget -relief]
	    }
	
	    if {[$w cget -state] ne "disabled"} {
			set Priv(buttonWindow) $w
			$w configure -relief sunken -state active
			set Priv($w,prelief) sunken
	
			# If this button has a repeatdelay set up, get it going with an after
			after cancel $Priv(afterId)
			set delay [$w cget -repeatdelay]
			set Priv(repeated) 0
			if {$delay > 0} {
			    set Priv(afterId) [after $delay [list tk::ButtonAutoInvoke $w]]
			}
	    }
	}
}

# ::tk::ButtonUp --
# The procedure below is invoked when the mouse button is released
# in a button widget.  It restores the button's relief and invokes
# the command as long as the mouse hasn't left the button.
#
# Arguments:
# w -		The name of the widget.

proc ::tk::ButtonUp w {
    variable ::tk::Priv
	catch {
		if {$Priv(buttonWindow) eq $w} {
			set Priv(buttonWindow) ""
		
			# Restore the button's relief if it was cached.
		
			if {[info exists Priv($w,relief)]} {
			    if {[info exists Priv($w,prelief)] && \
				    $Priv($w,prelief) eq [$w cget -relief]} {
				$w configure -relief $Priv($w,relief)
			    }
			    unset -nocomplain Priv($w,relief) Priv($w,prelief)
			}
		
			# Clean up the after event from the auto-repeater
			after cancel $Priv(afterId)
		
			if {$Priv(window) eq $w && [$w cget -state] ne "disabled"} {
			    $w configure -state normal
		
			    # Only invoke the command if it wasn't already invoked by the
			    # auto-repeater functionality
			    if { $Priv(repeated) == 0 } {
				uplevel #0 [list $w invoke]
			    }
			}
	    }
	}
}

